// transportserver.cpp : 
//

#include "stdafx.h"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <vector>
#include "transport.h"

class TrAsyncSocketClient;

int exit_flag = 0;

/*
 * This is only a simple demo, here assume just two clients
 * can connect to the server, the server then start these  
 * clients to traverse NATs.
 */
TrAsyncSocketClient *client1 = NULL;
TrAsyncSocketClient *client2 = NULL;

class TrAsyncSocketClient : public TrAsyncSocket {
public:
	TrAsyncSocketClient() : TrAsyncSocket(TR_SOCK_STREAM){
		cid_ = 0;
		connected_ = false;
	}

	TrAsyncSocketClient(TrAsyncSocketHelper *helper) : TrAsyncSocket(helper){
		cid_ = 0;
		connected_ = false;
	}

	~TrAsyncSocketClient(){ }

	void OnRead(char * data, int len){ 
		if(len < 0) return;

		printf("Recv %s len %d\n", data, len);

		/*
		 * Transfer candidates info between two clients
		 */
		if(cid_ == 1 && client2 && client2->connected_){
			printf("Trans data from client1 ==> client2\n");
			client2->Write(data, len);
		}
		else if(cid_ == 2 && client1 && client1->connected_){
			printf("Trans data from client2 ==> client1\n");
			client1->Write(data, len);
		}

		return;

	}

	virtual void OnSignalEvent(State s){
		switch(s){
			case STATE_CLOSED:
				printf("Client[%d] STATE_CLOSED\n", cid_);
				connected_ = false;
				break;
			case STATE_OPEN:
				printf("Client[%d] STATE_OPEN\n", cid_);
				/*
				 * A client has connected to the server so mark it
				 */
				connected_ = true;
				break;
		}
	}

public:
	int cid_;	
	bool connected_;
};

class TrAsyncSocketServer : public TrAsyncSocket {
public:
	TrAsyncSocketServer() : TrAsyncSocket(TR_SOCK_STREAM){}
	~TrAsyncSocketServer(){}

	void OnRead(char * data, int len){ 

		printf("Server received data???\n");
		return;

	}

	virtual void OnSignalEvent(State s){

		switch(s){
			case STATE_CLOSED:
				printf("Server STATE_CLOSED\n");
				break;
			case STATE_OPEN:
				printf("Server STATE_OPEN\n");
				break;
		}
	}

	virtual void OnAccept(TrAsyncSocketHelper *helper, TrSocketAddress remote_addr){

		printf("Client connected: %s:%d\n", remote_addr.host, remote_addr.port);

		/*
		 * A client want to connect to the server and SDK report this event to App
		 * App must create a new TrAsyncSocket and pass the helper return to SDK
		 * the helper object is only useful for the SDK 
		 */
		if(!client1){
			client1 = new TrAsyncSocketClient(helper);
			client1->cid_ = 1;
		}else if(!client2){
			client2 = new TrAsyncSocketClient(helper);
			client2->cid_ = 2;
		}
	}

};

int main(int argc, char* argv[])
{
	TrAsyncSocketServer *svr_socket;
	char ch;

	/*
	 * Before do anything by the SDK 
	 * Firstly please initialize it
	 */
	Tr_Init(NULL, NULL);

	/*
	 * Start STUN server on local host and listen to port 7000
	 */
	if(Tr_StartStunServer(7000)==T_ERROR)
		printf("Start STUN Server Failed");
	else
		printf("STUN Server listen at 7000\n");

	/*
	 * Start Relay server on local host and listen to port 5000
	 */
	if(Tr_StartRelayServer(5000)==T_ERROR)
		printf("Start Relay Server Failed");
	else
		printf("Relay Server listen at 5000\n");

	/*
	 * Start controling server and listen to port 9000
	 * Waiting clients to connect to here
	 */
	svr_socket = new TrAsyncSocketServer();	
	svr_socket->Bind(9000);
	svr_socket->Listen(5);

	while(1){

		printf("Enter q to exit program\n c to connect 2 clients:\n");
		ch = fgetc(stdin);
		if (ch == 'q'){
			exit_flag = 1;
			break;
		}else if (ch == 'c'){
			char cmd[32] = {0};

			sprintf_s(cmd, "%s", "START");

			if(!client1 || !client2) continue;

			/*
			 * Very simple control signal only one msg: START
			 * Tell two clients to begin connecting to each other
			 */
			client1->Write(cmd, (int)strlen(cmd)+1);
			client2->Write(cmd, (int)strlen(cmd)+1);
		}
		
	}

	client1->Close();
	client2->Close();

	if(client1) delete client1;
	if(client2) delete client2;

	Tr_Destroy();

	return 0;
}

